// import packages from d3
import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;
  
// defining and loading the data into a variable
  const speciesValue=(d)=>d.species;

//   defining the start and end value of x - axis
  const my = (selection) => {
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);
// creating a linear relation to data and defining the starting 
//     and ending value of y-axis
    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);
    
	
// giving colors 
        const marker = (d) => {
        	switch(d){
            case 'setosa':
              return 'red';
            case 'versicolor':
              return 'black';
            default:
              return 'blue';
          }
        }
        
// mapping the colors
    const marks = data.map((d) => ({
      color:marker(speciesValue(d)),
      x: x(xValue(d)),
      y: y(yValue(d)),
      
    }));
    
    
// plot the graph 
    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join('circle')
      .attr('cx', (d) => d.x)
      .attr('cy', (d) => d.y)
      .attr('r', radius).attr('fill',(d)=>d.color);

    // TODO make it idempotent
    // We'll do this live on May 22, 2021!
    // Tune in here: https://youtu.be/U8x862hBk4s

// adding the y-axis and the limits of it
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));
// adding the x-axis and the limits
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x));
    
		
  };

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
